local _, RS = ...
RS.Conditions = {}
local Conditions = RS.Conditions

-- local references - WoW and Lua
local IsSwimming = IsSwimming
local IsFlying = IsFlying
local UnitName = UnitName
local UnitPlayerOrPetInParty = UnitPlayerOrPetInParty
local bit_band = bit.band

-- local references - Addon
local Events        = nil
local addonSettings = nil
local Variables     = nil
local playerName    = UnitName("player")
local bossList      = nil
local playerSpellEffects = nil

function Conditions:OnLoad()
	Events = RS.Events
	playerSpellEffects = Events.playerSpellEffects
	addonSettings = ReactionsSave
	Variables = RS.Variables
	bossList = RS.Common.bossList
end

-- types of conditions used to check if certain conditions actually need to be checked
local CONDITION_TYPE_MISCELLANEOUS = 10001
local CONDITION_TYPE_COMBAT        = 10002
local CONDITION_TYPE_PARTY         = 10003
local CONDITION_TYPE_PLACE         = 10004
local CONDITION_TYPE_PLAYER        = 10005
local CONDITION_TYPE_GROUP_MEMBER  = 10006
local CONDITION_TYPE_CASTER        = 10007
local CONDITION_TYPE_TARGET        = 10008
local CONDITION_TYPE_AFFECTED      = 10009

----------------------------------------------------------------------------------------------------
-- check if a reaction can be used based on set conditions
----------------------------------------------------------------------------------------------------
local conditionFriendCache
local conditionGuildCache

local function CheckUnitConditions(conditionTable, name, prefix)
	if not name then
		return true
	end

	local unitid
	local targetIsNpc -- also used to skip guild/friend checking if possible
	local unitid = Variables:FindUnitID(name)
	if unitid then
		targetIsNpc = (not UnitIsPlayer(unitid))

		if UnitIsEnemy("player", unitid) then
			if conditionTable[prefix.."HostileReaction"] then return false end
		elseif UnitCanAttack("player", unitid) then
			if conditionTable[prefix.."NeutralReaction"] then return false end
		else
			if conditionTable[prefix.."FriendlyReaction"] then return false end
		end

		if not targetIsNpc then
			if name == playerName then
				if conditionTable[prefix.."Self"] then return false end
			else
				if conditionTable[prefix.."NotSelf"] then return false end
				if conditionTable[prefix.."Player"] then return false end
			end

			local _, race = UnitRace(unitid)
			if conditionTable[prefix..race] then return false end
		else
			-- pets don't count as being/not being a mob
			if UnitIsUnit(unitid, "pet") then
				if conditionTable[prefix.."MyPet"] then return false end
			elseif UnitPlayerOrPetInParty(unitid) then
				if conditionTable[prefix.."NotMyPet"] then return false end
			else
				if conditionTable[prefix.."Mob"] then return false end
			end
			if conditionTable[prefix.."NotSelf"] then return false end

			local classification = bossList[UnitName(unitid)] and "worldboss" or UnitClassification(unitid)
			if classification then -- not sure if it can be nil
				if conditionTable[prefix..classification] then return false end
			end
			if conditionTable[prefix..UnitCreatureType(unitid)] then return false end
		end

		local sex = UnitSex(unitid)
		if  (sex == 2 and conditionTable[prefix.."SexMale"])
		 or (sex == 3 and conditionTable[prefix.."SexFemale"])
		 or (sex == 1 and conditionTable[prefix.."SexUnknown"]) then
			return false
		end
	else
		local flags = (name == Events.sourceName and Events.sourceFlags) or (name == Events.destName and Events.destFlags) or nil
		if flags then
			-- special cases where a caster's hostility can be found without anyone looking at them
			if bit_band(flags, COMBATLOG_OBJECT_REACTION_FRIENDLY) == COMBATLOG_OBJECT_REACTION_FRIENDLY then
				if conditionTable[prefix.."FriendlyReaction"] then return false end
			elseif bit_band(flags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE then
				if conditionTable[prefix.."HostileReaction"] then return false end
			else
				if conditionTable[prefix.."NeutralReaction"] then return false end
			end
		end
	end

	-- focus conditions
	if UnitExists("focus") and UnitName("focus") == name then
		if conditionTable[prefix.."Focus"] then return false end
	else
		if conditionTable[prefix.."NotFocus"] then return false end
	end

	-- friend/guild conditions
	if not targetIsNpc and playerName ~= name then
		local checkFriends = conditionTable[prefix.."Friend"] or conditionTable[prefix.."NotFriend"]
		local checkGuild = (conditionTable[prefix.."Guild"] or conditionTable[prefix.."NotGuild"]) and IsInGuild()
		if checkFriends or checkGuild then
			local isFriend = conditionFriendCache[name]
			local isGuildMember = conditionGuildCache[name]

			if checkFriends and isFriend == nil then
				for i=1,C_FriendList.GetNumFriends() do
					if name == C_FriendList.GetFriendInfoByIndex(i).name then
						isFriend = true
						break
					end
				end
				conditionFriendCache[name] = isFriend or false
			end
			if checkGuild and isGuildMember == nil then
				for i=1,GetNumGuildMembers() do
					if name == GetGuildRosterInfo(i) then
						isGuildMember = true
						break
					end
				end
				conditionGuildCache[name] = isGuildMember or false
			end

			-- special handling for guild/friends - if they're allowed on any of the options then
			-- they're accepted at targets
			if not ( (isGuildMember and not conditionTable[prefix.."Guild"])
				or (not isGuildMember and not conditionTable[prefix.."NotGuild"])
				or (isFriend and not conditionTable[prefix.."Friend"])
				or (not isFriend and not conditionTable[prefix.."NotFriend"]) ) then
				return false
			end
		end
	end

	-- if information wasn't known about them, disqualify them
	if not unitid then
		local search = "^"..prefix.."."
		local exception1, exception2, exception3, exception4 = "Guild$", "Friend$", "Reaction$", "Focus$"
		for k in pairs(conditionTable) do
			k = tostring(k)
			if (k:find(search)) and not (k:find(exception1)) and not (k:find(exception2)) and not (k:find(exception3)) and not (k:find(exception4)) then
				return false
			end
		end
	end

	return true
end

-- Something being in the table means it's NOT allowed - done this way to save memory and to skip
-- checking since most cases will probably allow everything so the table won't even need to exist.
function Conditions.ReactionConditionsUsable(conditionTable, usableCombat, currentForm, hitType, spellId)
	if not conditionTable then
		return true
	end

	-- combat and hit/miss conditions
	if conditionTable[CONDITION_TYPE_COMBAT] then
		if conditionTable[usableCombat] then
			return false
		end
		if hitType then
			if conditionTable[hitType] then
				return false
			end
		elseif conditionTable["Normal"] then
			return false
		end
	end

	-- caster/target/group unit conditions
	local Variables = Variables
	if (conditionTable[CONDITION_TYPE_TARGET] and not CheckUnitConditions(conditionTable, Variables.extraTargetName or Variables.targetName or (Variables.casterName == playerName and playerName) or nil, "Target"))
	or (conditionTable[CONDITION_TYPE_GROUP_MEMBER]  and not CheckUnitConditions(conditionTable, Variables.memberName, "Group"))
	or (conditionTable[CONDITION_TYPE_CASTER] and not CheckUnitConditions(conditionTable, Variables.casterName, "Caster")) then
		return false
	end

	-- party conditions
	if conditionTable[CONDITION_TYPE_PARTY] then
		if UnitExists("pet") then
			if conditionTable["HavePet"] then return false end
		else
			if conditionTable["HaveNoPet"] then return false end
		end

		if IsInGroup() then
			if IsInRaid() then
				if conditionTable["Raid"] then return false end
				if IsRaidOfficer() then
					if conditionTable["GroupLeader"] then return false end
				else
					if conditionTable["NotGroupLeader"] then return false end
				end
			else
				if conditionTable["Party"] then return false end
				if UnitIsGroupLeader("player") then
					if conditionTable["GroupLeader"] then return false end
				else
					if conditionTable["NotGroupLeader"] then return false end
				end
			end
			local _, masterLooter = GetLootMethod()
			if masterLooter == 0 then
				if conditionTable["MasterLooter"] then return false end
			else
				if conditionTable["NotMasterLooter"] then return false end
			end
		elseif conditionTable["Ungrouped"] then
			return false
		end
	end

	if conditionTable[CONDITION_TYPE_AFFECTED] then
		local effects = playerSpellEffects
		for i=1,17 do
			if effects[i] == 0 then
				if conditionTable[-i] then
					return false
				end
			elseif conditionTable[i] then
				return false
			end
		end
	end

	-- player conditions
	if conditionTable[CONDITION_TYPE_PLAYER] then
		if conditionTable[currentForm] then return false end

		if IsMounted() then
			if conditionTable["CharMounted"] then return false end
		else
			if conditionTable["CharNotMounted"] then return false end
		end
		if IsSwimming() then
			if conditionTable["CharSwimming"] then return false end
		else
			if conditionTable["CharNotSwimming"] then return false end
		end
		if IsFlying() then
			if conditionTable["CharFlying"] then return false end
		else
			if conditionTable["CharNotFlying"] then return false end
		end
		if IsStealthed() then
			if conditionTable["CharStealthed"] then return false end
		else
			if conditionTable["CharNotStealthed"] then return false end
		end

		local health = floor(UnitHealth("player") / UnitHealthMax("player") * 100)
		if health <= addonSettings.lowHealthBegin then
			if conditionTable["CharLowHealth"] then return false end
		elseif health >= addonSettings.lowHealthEnd then
			if conditionTable["CharHighHealth"] then return false end
		else
			if conditionTable["CharMediumHealth"] then return false end
		end

		local Power = floor(UnitPower("player", 0) / UnitPowerMax("player", 0) * 100)
		if Power <= addonSettings.lowManaBegin then
			if conditionTable["CharLowPower"] then return false end
		elseif Power >= addonSettings.lowManaEnd then
			if conditionTable["CharHighPower"] then return false end
		else
			if conditionTable["CharMediumPower"] then return false end
		end
	end

	-- location conditions
	if conditionTable[CONDITION_TYPE_PLACE] then
		local _, instanceType = IsInInstance()
		if instanceType == "arena" then
			if conditionTable["Arenas"] then return false end
		elseif instanceType == "pvp" then
			if conditionTable["Battlegrounds"] then return false end
		elseif instanceType ~= "none" then
			if conditionTable["Instances"] then return false end
		else
			if conditionTable["WorldZones"] then return false end
		end

		if IsIndoors() then
			if conditionTable["Indoors"] then return false end
		else
			if conditionTable["Outdoors"] then return false end
		end

		if IsResting() then
			if conditionTable["CharResting"] then return false end
		else
			if conditionTable["CharNotResting"] then return false end
		end

		local pvpStatus = (GetZonePVPInfo())
		if pvpStatus and conditionTable[pvpStatus] then
			return false
		end
	end

	-- miscellaneous conditions
	if conditionTable[CONDITION_TYPE_MISCELLANEOUS] then
		-- spell ranks
		if spellId then
			local _, rank = GetSpellInfo(spellId)
			if rank and conditionTable[rank] then
				return false
			end
		end

		-- time conditions
		local realHour = date("*t").hour
		if realHour < 5 or realHour >= 21  then
			if conditionTable["RealNight"] then return false end
		elseif realHour < 12 then
			if conditionTable["RealMorning"] then return false end
		elseif realHour < 17 then
			if conditionTable["RealAfternoon"] then return false end
		else
			if conditionTable["RealEvening"] then return false end
		end

		local gameHour = GetGameTime()
		if gameHour < 5 or gameHour >= 21 then
			if conditionTable["Night"] then return false end
		elseif gameHour < 12 then
			if conditionTable["Morning"] then return false end
		elseif gameHour < 17 then
			if conditionTable["Afternoon"] then return false end
		else
			if conditionTable["Evening"] then return false end
		end
	end

	return true
end

-- check if a specific role item is being worn
function Conditions.ReactionWearingRoleItem(number)
	local item = addonSettings.roleItems[number]
	if item then
		local link = GetInventoryItemLink("player", item[4])
		if link and link:match("%[(.-)]") == item[1] then
			return true
		end
		if item[5] then
			link = GetInventoryItemLink("player", item[5])
			if link and link:match("%[(.-)]") == item[1] then
				tanking = true
			end
		end
		return nil
	end
	return true
end
